#!/usr/bin/perl
######################################################################
#   File          : Huawei_MP10FHD_Split.pl
#   Author(s)     : McSpoon
#   Description   : Unpack a Huawei 'update.app' file.
#   Last Modified : v0.04 beta (2013-02-28)
#   By            : Nafanovich
#   Comment       : For Huawei MediaPad 10 FHD
#   Site          : http://www.273k.info/
#   Support Me    : http://db.tt/jMKzYFg
######################################################################
 
use strict;
use warnings;

my $CRC_CHECK= -e "crc" && -x _;
 
my %fileHash=(
	"\x4D\x44\x35\x52\x53\x41\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","md5rsa.bin",
	"\x43\x52\x43\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","crc.bin",
	"\x50\x54\x41\x42\x4C\x45\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","ptable.bin",
	"\x46\x41\x53\x54\x42\x4F\x4F\x54\x00\x00\x00\x00\x00\x00\x00\x00","fastboot.img",
	"\x42\x4F\x4F\x54\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","boot.img",
	"\x52\x45\x43\x4F\x56\x45\x52\x59\x00\x00\x00\x00\x00\x00\x00\x00","recovery.img",
	"\x53\x59\x53\x54\x45\x4D\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","system.img",
	"\x55\x53\x45\x52\x44\x41\x54\x41\x00\x00\x00\x00\x00\x00\x00\x00","userdata.bin",
	"\x4C\x4F\x47\x4F\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","logo.bin",
	"\x43\x41\x43\x48\x45\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","cashe.bin",
	"\x43\x55\x53\x54\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","custom.img",
	"\x4D\x4F\x44\x45\x4D\x49\x4D\x41\x47\x45\x00\x00\x00\x00\x00\x00","modemimage.bin",
	"\x4D\x4F\x44\x45\x4D\x4E\x56\x00\x00\x00\x00\x00\x00\x00\x00\x00","modemnv.bin",
	"\x4F\x45\x4D\x49\x4E\x46\x4F\x00\x00\x00\x00\x00\x00\x00\x00\x00","oeminfo.bin",
	"\x52\x4F\x55\x4E\x44\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00","round.bin",
);

my $unknown_count=0;
$|++;
use constant UINT_SIZE => 4; # Unsigned integers are 4 bytes.
my $FILENAME = undef;
$FILENAME = "update.app"; # File to Split.

open(INFILE, $FILENAME) or die "Cannot open $FILENAME: $!\n";
binmode INFILE;
my $fileLoc=0;
my $BASEPATH = "output/"; # We'll dump the files into a folder called "output".
mkdir $BASEPATH;
while (!eof(INFILE)) {
	$fileLoc=&find_next_file($fileLoc);
	seek(INFILE, $fileLoc, 0);
	$fileLoc=&dump_file();
}
close INFILE;

# Find the next file block in the main file
sub find_next_file {
	my ($_fileLoc) = @_;
	my $_buffer = undef;
	my $_skipped=0;
	read(INFILE, $_buffer, UINT_SIZE);
	while ($_buffer ne "\x55\xAA\x5A\xA5" && !eof(INFILE)) {
		read(INFILE, $_buffer, UINT_SIZE);
		$_skipped+=UINT_SIZE;
	}
	return($_fileLoc + $_skipped);
}
 
# Unpack a file block and output the payload to a file.
sub dump_file {
	my $buffer = undef;
	my $outfilename = undef;
	my $fileSeq;
	my $calculatedcrc = undef;
	my $sourcecrc = undef;
	my $fileChecksum;
	read(INFILE, $buffer, UINT_SIZE); # Packet Identifier
	unless ($buffer eq "\x55\xAA\x5A\xA5") {
		die "Unrecognised file format. Wrong identifier.\n";
	}
	read(INFILE, $buffer, UINT_SIZE); # Packet Length.
	my ($headerLength) = unpack("V", $buffer);
	read(INFILE, $buffer, 4); # Always 1
	read(INFILE, $buffer, 8); # Hardware ID
	read(INFILE, $buffer, 4); # Always FFFFFFFF
	read(INFILE, $buffer, UINT_SIZE); # Data file length
	my ($dataLength) = unpack("V", $buffer);
	read(INFILE, $buffer, 16); # File date
	read(INFILE, $buffer, 16); # File time
	read(INFILE, $fileSeq, 16); # The word INPUT ?
	if (exists($fileHash{$fileSeq})) {
		$outfilename=$fileHash{$fileSeq};
	} else {
		$outfilename="unknown_file.$unknown_count";
		$unknown_count++;
	}
	read(INFILE, $buffer, 16); # Blank
	read(INFILE, $buffer, 2); # Checksum of the header maybe?
	read(INFILE, $buffer, 2); # Always 1?
	read(INFILE, $buffer, 2); # Blank
	# Grab the checksum of the file
	read(INFILE, $fileChecksum, $headerLength-98);
	$sourcecrc=slimhexdump($fileChecksum);
	# Dump the payload.
	read(INFILE, $buffer, $dataLength);
	open(OUTFILE, ">$BASEPATH$outfilename") or die "Unable to create $outfilename: $!\n";
	binmode OUTFILE;
	print OUTFILE $buffer;
	close OUTFILE;
	$calculatedcrc=`./crc $BASEPATH$outfilename` if $CRC_CHECK;
	chomp($calculatedcrc) if $CRC_CHECK;
	print STDOUT "Extracted $outfilename";
	print "\n" if !$CRC_CHECK;
	if($CRC_CHECK){
		if ($calculatedcrc eq $sourcecrc) {
			print " - CRC Okay\n";
		} else {
			print " - ERROR: CRC did not match\n";
		}
	}
	# Ensure we finish on a 4 byte boundary alignment.
	my $remainder = UINT_SIZE - (tell(INFILE) % UINT_SIZE);
	if ($remainder < UINT_SIZE) {
		# We can ignore the remaining padding.
		read(INFILE, $buffer, $remainder);
	}
	return (tell(INFILE));
}

sub hexdump () {
	my $num=0;
	my $i;
	my $rhs;
	my $lhs;
	my ($buf) = @_;
	my $ret_str="";
	# This loop is to process each character at a time.
	foreach $i ($buf =~ m/./gs) {
		$lhs .= sprintf(" %02X",ord($i));
		if ($i =~ m/[ -~]/) {
			$rhs .= $i;
		} else {
			$rhs .= ".";
		}
		$num++;
		if (($num % 16) == 0) {
			$ret_str.=sprintf("%-50s %s\n",$lhs,$rhs);
			$lhs="";
			$rhs="";
		}
	}
	if (($num % 16) != 0) {
		$ret_str.=sprintf("%-50s %s\n",$lhs,$rhs);
	}
	return ($ret_str);
}

sub slimhexdump () {
	my $i;
	my ($buf) = @_;
	my $ret_str="";
	# This loop is to process each character at a time.
	foreach $i ($buf =~ m/./gs) {
		$ret_str .= sprintf("%02X",ord($i));
	}
	return ($ret_str);
}